//=============================================================================
// AltSaveScreenCyan.js
//=============================================================================

/*:
 * @plugindesc Alternative save/load screen layout.
 * @author Yoji Ojima modified by liply
 *
 * @help This plugin does not provide plugin commands.
 *
 * @param Background Variable
 * @desc セーブ画面のバックグラウンドイメージの変数です。save-background-[変数]という名前の画像が使用されます。
 * @default 10
 */

(function() {

    var parameters = PluginManager.parameters('AltSaveScreenCyan');
    var backgroundVariable = parseInt(parameters['Background Variable']);

    var _Scene_File_create = Scene_File.prototype.create;
    Scene_File.prototype.create = function() {
        _Scene_File_create.call(this);
        this._listWindow.height = this._listWindow.fittingHeight(14);
        this._listWindow.width = Graphics.boxWidth / 2;
        this._listWindow.setBackgroundType(1);

        var width = Graphics.boxWidth / 2;
        var height = this._listWindow.fittingHeight(2);
        var x = width;
        var y = Graphics.boxHeight - height;
        this._statusWindow = new Window_SavefileStatus(x, y, width, height);
        this._statusWindow.setMode(this.mode());
        this._listWindow.statusWindow = this._statusWindow;
        this._listWindow.callUpdateHelp();
        this.addWindow(this._statusWindow);
    };

    var _Scene_File_createBackground = Scene_File.prototype.createBackground;
    Scene_File.prototype.createBackground = function(){
        if(this.mode() === 'save'){
            var bgVar = $gameVariables.value(backgroundVariable);
            this._backgroundSprite = new Sprite();
            this._backgroundSprite.bitmap = ImageManager.loadPicture('save-' + bgVar);
            this.addChild(this._backgroundSprite);
        }else{
            _Scene_File_createBackground.apply(this, arguments);
        }
    };

    var _Scene_File_start = Scene_File.prototype.start;
    Scene_File.prototype.start = function() {
        _Scene_File_start.call(this);
        this._listWindow.ensureCursorVisible();
        this._listWindow.callUpdateHelp();
        this._listWindow.reselect();
    };

    Window_SavefileList.prototype.windowWidth = function() {
        return Graphics.boxWidth;
    };

    Window_SavefileList.prototype.maxCols = function() {
        return 2;
    };

    Window_SavefileList.prototype.numVisibleRows = function() {
        return 6;
    };

    Window_SavefileList.prototype.spacing = function() {
        return 8;
    };

    Window_SavefileList.prototype.itemHeight = function() {
        return this.lineHeight() * 2;
    };

    var _Window_SavefileList_callUpdateHelp =
            Window_SavefileList.prototype.callUpdateHelp;
    Window_SavefileList.prototype.callUpdateHelp = function() {
        _Window_SavefileList_callUpdateHelp.call(this);
        if (this.active && this.statusWindow) {
            this.statusWindow.setId(this.index() + 1);
        }
    };

    function Window_SavefileStatus() {
        this.initialize.apply(this, arguments);
    }

    Window_SavefileStatus.prototype = Object.create(Window_Base.prototype);
    Window_SavefileStatus.prototype.constructor = Window_SavefileStatus;

    Window_SavefileStatus.prototype.initialize = function(x, y, width, height) {
        Window_Base.prototype.initialize.call(this, x, y, width, height);
        this.setBackgroundType(2);
        this._id = 1;
    };

    Window_SavefileStatus.prototype.setMode = function(mode) {
        this._mode = mode;
    };

    Window_SavefileStatus.prototype.setId = function(id) {
        this._id = id;
        this.refresh();
    };

    Window_SavefileStatus.prototype.refresh = function() {
        this.contents.clear();
        var id = this._id;
        var valid = DataManager.isThisGameFile(id);
        var info = DataManager.loadSavefileInfo(id);
        var rect = this.contents.rect;
        this.resetTextColor();
        if (this._mode === 'load') {
            this.changePaintOpacity(valid);
        }
        if (info) {
            this.changePaintOpacity(valid);
            this.drawContents(info, rect, valid);
            this.changePaintOpacity(true);
        }
    };

    Window_SavefileStatus.prototype.drawContents = function(info, rect, valid) {
        this.drawText(info.title, rect.x, rect.y, rect.width, 'right');
        this.drawText(info.playtime, rect.x, rect.y + this.lineHeight(), rect.width, 'right');
    };

})();
